package jp.kiroru_inc.samples.oredialer;

import java.text.SimpleDateFormat;

import android.content.ContentUris;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.provider.CallLog.Calls;
import android.provider.ContactsContract.Contacts;
import android.provider.ContactsContract.PhoneLookup;
import android.support.v4.app.ListFragment;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.CursorAdapter;
import android.widget.ImageView;
import android.widget.TextView;

public class CallLogFragment extends ListFragment {

    private String[] projection = {
            Calls._ID,          // 0
            Calls.TYPE,         // 1
            Calls.DATE,         // 2
            Calls.CACHED_NAME,  // 3
            Calls.NUMBER,       // 4
    };
    
    private static int INDEX_TYPE         = 1;
    private static int INDEX_DATE         = 2;
    private static int INDEX_CACHED_NAME  = 3;
    private static int INDEX_NUMBER       = 4;
    
    private static int MENU_GROUP_CALLLOG          = 2;
    
    private static int MENU_CALLLOG_DIAL           = 1;
    private static int MENU_CALLLOG_VIEW           = 2;
    private static int MENU_CALLLOG_INSERT_OR_EDIT = 3;
    
    private static CallLogFragment _instance = null;
    
    public static CallLogFragment create() {
        if (_instance == null) {
            _instance = new CallLogFragment();
        }
        return _instance;
    }
    
    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
    
        Cursor c = getActivity().managedQuery(Calls.CONTENT_URI, projection, null, null, Calls.DEFAULT_SORT_ORDER);
        MyAdapter adapter = new MyAdapter(getActivity(), c);
        setListAdapter(adapter);
        registerForContextMenu(getListView());
        
        getListView().setOnItemClickListener(new OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View v, int position, long id) {
                Uri uri = ContentUris.withAppendedId(Calls.CONTENT_URI, id);
                String[] projection = { Calls.NUMBER };
                Cursor c = getActivity().getContentResolver().query(uri, projection, null, null, null);
                if (c.moveToFirst()) {
                    String number = (String)c.getString(0);
                    c.close();
                    dial(number);
                }                
            }
        });
    }
    
    @Override
    public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
        menu.add(MENU_GROUP_CALLLOG, MENU_CALLLOG_DIAL, 0, "ダイアルパッド");
        menu.add(MENU_GROUP_CALLLOG, MENU_CALLLOG_VIEW, 1, "詳細");
        menu.add(MENU_GROUP_CALLLOG, MENU_CALLLOG_INSERT_OR_EDIT, 2, "新規登録または既存コンタクトに追加");
    };

    @Override
    public boolean onContextItemSelected(MenuItem item) {
        if (item.getGroupId() != MENU_GROUP_CALLLOG) {
            return false;
        }
        long id = ((AdapterContextMenuInfo)item.getMenuInfo()).id;
        int menuId = item.getItemId();
        if (menuId == MENU_CALLLOG_DIAL || menuId == MENU_CALLLOG_VIEW || 
            menuId == MENU_CALLLOG_INSERT_OR_EDIT) {
            Uri uri = ContentUris.withAppendedId(Calls.CONTENT_URI, id);
            String[] projection = { Calls.NUMBER };
            Cursor c = getActivity().getContentResolver().query(uri, projection, null, null, null);
            if (c.moveToFirst()) {
                String number = (String)c.getString(0);
                c.close();
                if (number != null && !number.equals("") && !number.startsWith("-")) {
                    if (menuId == MENU_CALLLOG_DIAL) {
                        dial(number);
                    } else if (menuId == MENU_CALLLOG_VIEW) {
                        view(number);
                    } else {
                        insert_or_edit(number);
                    }
                }
            }                
            return true;
        }
        return false;
    }

    private void dial(String number) {
        Uri telUri = Uri.parse("tel:" + number);
        Intent i = new Intent(Intent.ACTION_DIAL);
        i.setData(telUri);
        i.putExtra("number", number);
        startActivity(i);
    }
    
    private void view(String number) {
        Uri phoneLookupUri = Uri.withAppendedPath(PhoneLookup.CONTENT_FILTER_URI, Uri.encode(number));
        String[] projection = { PhoneLookup.LOOKUP_KEY };
        Cursor c = getActivity().getContentResolver().query(phoneLookupUri, projection, null, null, null);
        if (c.moveToFirst()) {
            String lookupKey = (String)c.getString(0);
            c.close();
            Uri lookupUri = Uri.withAppendedPath(Contacts.CONTENT_LOOKUP_URI, lookupKey);
            Intent i = new Intent(Intent.ACTION_VIEW);
            i.setDataAndType(lookupUri, Contacts.CONTENT_ITEM_TYPE);
            startActivity(i);
        }
    }
    
    private void insert_or_edit(String number) {
        Intent i = new Intent(Intent.ACTION_INSERT_OR_EDIT);
        i.setType(Contacts.CONTENT_ITEM_TYPE);
        i.putExtra("phone", number);
        startActivity(i);
    }
    
    private class MyAdapter extends CursorAdapter {
        public MyAdapter(Context context, Cursor c) {
            super(context, c);
        }

        @Override
        public View newView(Context context, Cursor cursor, ViewGroup parent) {
            return LayoutInflater.from(context).inflate(R.layout.calllog_row, parent, false);
        }
        
        @Override
        public void bindView(View view, Context context, Cursor cursor) {
            // タイプ
            int resId;
            int type = cursor.getInt(INDEX_TYPE);
            if (type == Calls.INCOMING_TYPE) {
                resId = android.R.drawable.sym_call_incoming;
            } else if (type == Calls.OUTGOING_TYPE) {
                resId = android.R.drawable.sym_call_outgoing;
            } else { // (type == Calls.MISSED_TYPE) 
                resId = android.R.drawable.sym_call_missed;
            }
            ImageView iv = (ImageView)view.findViewById(R.id.image);
            iv.setImageResource(resId);
            
            // 時刻
            long date = cursor.getLong(INDEX_DATE);
            String text1 = getFormatedDate(date);
            TextView tv1 = (TextView) view.findViewById(R.id.text1);
            tv1.setText(text1);
            
            // 名前
            String text2 = cursor.getString(INDEX_CACHED_NAME);
            TextView tv2 = (TextView) view.findViewById(R.id.text2);
            tv2.setText(text2);

            // 電話番号
            String text3 = cursor.getString(INDEX_NUMBER);
            TextView tv3 = (TextView) view.findViewById(R.id.text3);
            tv3.setText(text3);
        }
    }
    
    private final long ONEMINUTE = 60 * 1000;
    private final long ONEHOUR = 60 * ONEMINUTE;
    private final long ONEDAY = 24 * ONEHOUR;
    private final long ONEMONTH = 30 * ONEDAY;
    
    private String getFormatedDate(long date) {
        long now = System.currentTimeMillis();
        long diff = now - date;
        if (diff < ONEHOUR) {
            long a = diff / ONEMINUTE;
            return String.format("%d 分前", a);
        }
        else if (diff < ONEDAY) {
            long a = diff / ONEHOUR;
            return String.format("%d 時間前", a);
        }
        else if (diff < (ONEDAY * 2)) {
            return "昨日";
        }
        else if (diff < ONEMONTH) {
            long a = diff / ONEDAY;
            return String.format("%d 日前", a);
        }
        
        SimpleDateFormat sdf1 = new SimpleDateFormat();
        sdf1.applyPattern("yyyy/MM/dd");
        return sdf1.format(date);
    }
}
