package jp.kiroru_inc.samples.oredialer;

import android.content.ContentUris;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Bundle;
import android.provider.ContactsContract.Contacts;
import android.provider.ContactsContract.Data;
import android.provider.ContactsContract.RawContacts;
import android.provider.ContactsContract.CommonDataKinds.Phone;
import android.provider.ContactsContract.CommonDataKinds.Photo;
import android.support.v4.app.ListFragment;
import android.view.ContextMenu;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.ContextMenu.ContextMenuInfo;
import android.widget.AdapterView;
import android.widget.CursorAdapter;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.AdapterView.OnItemClickListener;

public class ContactListFragment extends ListFragment {

    private String[] projection = {
            Phone._ID,           // 0
            Phone.PHOTO_ID,      // 1
            Phone.DISPLAY_NAME,  // 2
            Phone.NUMBER,        // 3
    };
    
    private static int INDEX_PHOTO_ID     = 1;
    private static int INDEX_DISPLAY_NAME = 2;
    private static int INDEX_NUMBER       = 3;

    private static int MENU_GROUP_CONTACT = 1;
    
    private static int MENU_CONTACT_DIAL = 1;
    private static int MENU_CONTACT_VIEW = 2;
    private static int MENU_CONTACT_EDIT = 3;
    
    private static ContactListFragment _instance = null;
    
    public static ContactListFragment create() {
        if (_instance == null) {
            _instance = new ContactListFragment();
        }
        return _instance;
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);

        Cursor c = getActivity().managedQuery(Phone.CONTENT_URI, projection, null, null, null);
        MyAdapter adapter = new MyAdapter(getActivity(), c);
        setListAdapter(adapter);
        registerForContextMenu(getListView());
    
        getListView().setOnItemClickListener(new OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View v, int position, long id) {
                Uri uri = ContentUris.withAppendedId(Phone.CONTENT_URI, id);
                String[] projection = { Phone.NUMBER };
                Cursor c = getActivity().getContentResolver().query(uri, projection, null, null, null);
                if (c.moveToFirst()) {
                    String number = (String)c.getString(0);
                    c.close();
                    dial(number);
                }                
            }
        });
    }
    
    @Override
    public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
        menu.add(MENU_GROUP_CONTACT, MENU_CONTACT_DIAL, 0, "ダイアルパッド");
        menu.add(MENU_GROUP_CONTACT, MENU_CONTACT_VIEW, 1, "詳細");
        menu.add(MENU_GROUP_CONTACT, MENU_CONTACT_EDIT, 2, "編集");
    };

    @Override
    public boolean onContextItemSelected(MenuItem item) {
        if (item.getGroupId() != MENU_GROUP_CONTACT) {
            return false;
        }
        long id = ((AdapterContextMenuInfo)item.getMenuInfo()).id;
        int menuId = item.getItemId();
        if (menuId == MENU_CONTACT_DIAL) {
            Uri uri = ContentUris.withAppendedId(Phone.CONTENT_URI, id);
            String[] projection = { Phone.NUMBER };
            Cursor c = getActivity().getContentResolver().query(uri, projection, null, null, null);
            if (c.moveToFirst()) {
                String number = (String)c.getString(0);
                c.close();
                dial(number);
            }                
            return true;
            
        } else if (menuId == MENU_CONTACT_VIEW) {
            Uri uri = ContentUris.withAppendedId(Phone.CONTENT_URI, id);
            String[] projection = { Phone.LOOKUP_KEY };
            Cursor c = getActivity().getContentResolver().query(uri, projection, null, null, null);
            if (c.moveToFirst()) {
                String lookupKey = (String)c.getString(0);
                c.close();
                view(lookupKey);
            }                
            return true;

        } else if (menuId == MENU_CONTACT_EDIT) {
            Uri uri = ContentUris.withAppendedId(Phone.CONTENT_URI, id);
            String[] projection = { Phone.RAW_CONTACT_ID };
            Cursor c = getActivity().getContentResolver().query(uri, projection, null, null, null);
            if (c.moveToFirst()) {
                String rawContactId = (String)c.getString(0);
                c.close();
                edit(rawContactId);
            }                
            return true;
        }
        return false;
    }
    
    private void dial(String number) {
        Uri telUri = Uri.parse("tel:" + number);
        Intent i = new Intent(Intent.ACTION_DIAL);
        i.setData(telUri);
        startActivity(i);
    }
    
    private void view(String lookupKey) {
        Uri lookupUri = Uri.withAppendedPath(Contacts.CONTENT_LOOKUP_URI, lookupKey);
        Intent i = new Intent(Intent.ACTION_VIEW);
        i.setDataAndType(lookupUri, Contacts.CONTENT_ITEM_TYPE);
        startActivity(i);
    }
    
    private void edit(String rawContactId) {
        Uri rawContactUri = Uri.withAppendedPath(RawContacts.CONTENT_URI, rawContactId);
        Intent i = new Intent(Intent.ACTION_EDIT);
        i.setDataAndType(rawContactUri, Contacts.CONTENT_ITEM_TYPE);
        startActivity(i);
    }
    
    private class MyAdapter extends CursorAdapter {
        public MyAdapter(Context context, Cursor c) {
            super(context, c);
        }

        @Override
        public View newView(Context context, Cursor cursor, ViewGroup parent) {
            return LayoutInflater.from(context).inflate(R.layout.contactlist_row, parent, false);
        }
        
        @Override
        public void bindView(View view, Context context, Cursor cursor) {
            // 写真
            ImageView iv = (ImageView) view.findViewById(R.id.image);
            byte[] photoData = null;
            String photoId = cursor.getString(INDEX_PHOTO_ID);
            if (photoId != null) {
                Uri uri = Uri.withAppendedPath(Data.CONTENT_URI, photoId);
                String[] projection = { Photo.PHOTO };
                Cursor c = context.getContentResolver().query(uri, projection, null, null, null);
                if (c.moveToFirst()) {
                    photoData = c.getBlob(0);
                }
                c.close();
            }
            if (photoData != null) {
                iv.setImageBitmap(BitmapFactory.decodeByteArray(photoData, 0, photoData.length));
            } else {
                iv.setImageResource(R.drawable.ic_contact_picture);
            }
            
            // 名前
            String text1 = cursor.getString(INDEX_DISPLAY_NAME);
            TextView tv1 = (TextView) view.findViewById(R.id.text1);
            tv1.setText(text1);

            // 電話番号
            String text2 = cursor.getString(INDEX_NUMBER);
            TextView tv2 = (TextView) view.findViewById(R.id.text2);
            tv2.setText(text2);
        }
    }
}
