package jp.kiroru_inc.samples.oredialer;

import android.content.Intent;
import android.media.AudioManager;
import android.media.ToneGenerator;
import android.net.Uri;
import android.os.Bundle;
import android.provider.ContactsContract.Contacts;
import android.support.v4.app.Fragment;
import android.telephony.PhoneNumberUtils;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnLongClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;

public class DialPadFragment extends Fragment {

    private static final int TONE_RELATIVE_VOLUME = 80;
    private Object mToneGeneratorLock = new Object();
    private ToneGenerator mToneGenerator;
    private EditText editNumber;
    private Button b_call;
    private Button b_plus;
    private String number = null;
    private static DialPadFragment _instance = null;
    
    public static DialPadFragment create() {
        if (_instance == null) {
            _instance = new DialPadFragment();
        }
        return _instance;
    }
    
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View v = inflater.inflate(R.layout.dialpad, container, false);
        return v;
    }
    
    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        
        // 入力ボタン
        Button b1 = (Button)getActivity().findViewById(R.id.button1);
        b1.setOnClickListener(new MyListener(ToneGenerator.TONE_DTMF_1, "1"));

        Button b2 = (Button)getActivity().findViewById(R.id.button2);
        b2.setOnClickListener(new MyListener(ToneGenerator.TONE_DTMF_2, "2"));

        Button b3 = (Button)getActivity().findViewById(R.id.button3);
        b3.setOnClickListener(new MyListener(ToneGenerator.TONE_DTMF_3, "3"));

        Button b4 = (Button)getActivity().findViewById(R.id.button4);
        b4.setOnClickListener(new MyListener(ToneGenerator.TONE_DTMF_4, "4"));

        Button b5 = (Button)getActivity().findViewById(R.id.button5);
        b5.setOnClickListener(new MyListener(ToneGenerator.TONE_DTMF_5, "5"));

        Button b6 = (Button)getActivity().findViewById(R.id.button6);
        b6.setOnClickListener(new MyListener(ToneGenerator.TONE_DTMF_6, "6"));

        Button b7 = (Button)getActivity().findViewById(R.id.button7);
        b7.setOnClickListener(new MyListener(ToneGenerator.TONE_DTMF_7, "7"));

        Button b8 = (Button)getActivity().findViewById(R.id.button8);
        b8.setOnClickListener(new MyListener(ToneGenerator.TONE_DTMF_8, "8"));

        Button b9 = (Button)getActivity().findViewById(R.id.button9);
        b9.setOnClickListener(new MyListener(ToneGenerator.TONE_DTMF_9, "9"));

        Button bs = (Button)getActivity().findViewById(R.id.button_star);
        bs.setOnClickListener(new MyListener(ToneGenerator.TONE_DTMF_S, "*"));

        Button b0 = (Button)getActivity().findViewById(R.id.button0);
        b0.setOnClickListener(new MyListener(ToneGenerator.TONE_DTMF_0, "0"));

        Button bp = (Button)getActivity().findViewById(R.id.button_pound);
        bp.setOnClickListener(new MyListener(ToneGenerator.TONE_DTMF_P, "#"));

        // 削除ボタン
        Button b_del = (Button)getActivity().findViewById(R.id.buttonDel);
        b_del.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                int len = editNumber.getText().length();
                if (len > 0) {
                    editNumber.getText().delete(len-1, len);
                }
            }
        });
        b_del.setOnLongClickListener(new OnLongClickListener() {
            @Override
            public boolean onLongClick(View v) {
                int len = editNumber.getText().length();
                editNumber.getText().delete(0, len);
                return true;
            }
        });
        
        // 発信ボタン
        b_call = (Button)getActivity().findViewById(R.id.buttonCall);
        b_call.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                String number = editNumber.getText().toString();
                call(number);
            }
        });

        // ＋ボタン
        b_plus = (Button)getActivity().findViewById(R.id.buttonPlus);
        b_plus.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                String number = editNumber.getText().toString();
                insert(number);
            }
        });
        
        // 電話番号表示欄
        editNumber = (EditText)getActivity().findViewById(R.id.editNumber);
        editNumber.addTextChangedListener(new TextWatcher() {
            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                updateUI();
            }
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {}

            @Override
            public void afterTextChanged(Editable s) {}
        });
        updateNumber();
    }
    
    @Override
    public void onPause() {
        super.onPause();
    
        // トーンジェネレータを解放
        synchronized(mToneGeneratorLock) {
            if (mToneGenerator != null) {
                mToneGenerator.release();
                mToneGenerator = null;
            }
        }
    }
    
    @Override
    public void onResume() {
        super.onResume();
        
        updateUI();
        
        // トーンジェネレータの生成
        synchronized(mToneGeneratorLock) {
            if (mToneGenerator == null) {
                try {
                    mToneGenerator = new ToneGenerator(
                            AudioManager.STREAM_MUSIC, TONE_RELATIVE_VOLUME);
                    getActivity().setVolumeControlStream(AudioManager.STREAM_MUSIC);
                } catch (RuntimeException e) {
                    mToneGenerator = null;
                }
            }
        }
    }

    // 電話番号を表示を更新する
    public void updateNumber(Intent intent) {        
        Uri uri = intent.getData();
        if (uri != null && "tel".equals(uri.getScheme())) {
            String data = uri.getSchemeSpecificPart();
            number = PhoneNumberUtils.extractNetworkPortion(data);
            
            updateNumber();
        }
    }

    private void updateNumber() {
        if (!TextUtils.isEmpty(number) && editNumber != null) {
            editNumber.getText().clear();
            editNumber.getText().append(number);
        }
    }

    // ボタンの有効／無効を制御する
    private void updateUI() {
        int len = editNumber.length();
        if (len > 0) {
            b_plus.setEnabled(true);
            b_call.setEnabled(true);
        } else {
            b_plus.setEnabled(false);
            b_call.setEnabled(false);
        }
    }

    private void call(String number) {
        Uri telUri = Uri.parse("tel:" + number);
        Intent i = new Intent(Intent.ACTION_CALL);
        i.setData(telUri);
        startActivity(i);
    }
    
    // 電話番号の新規登録
    private void insert(String number) {
        Intent i = new Intent(Intent.ACTION_INSERT);
        i.setData(Contacts.CONTENT_URI);
        i.putExtra("phone", number);
        startActivity(i);
    }
    
    // 入力ボタンを押したときのリスナー
    private class MyListener implements OnClickListener {
        private static final int TONE_LENGTH_MS = 150;
        private int tone;
        private String s;
        
        public MyListener(int tone, String s) {
            this.tone = tone;
            this.s = s;
        }
        
        @Override
        public void onClick(View v) {
            playTone(tone);
            editNumber.getText().append(s);
        }

        private void playTone(int tone) {
            synchronized(mToneGeneratorLock) {
                if (mToneGenerator != null) {
                    mToneGenerator.startTone(tone, TONE_LENGTH_MS);
                }
            }
        }
    }
}
