package jp.kiroru_inc.samples.oredialer;

import android.content.Intent;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.view.ViewPager;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.RadioGroup.OnCheckedChangeListener;

public class MainActivity extends FragmentActivity {

    private static int ids[] = {
        R.id.rb_1, R.id.rb_2, R.id.rb_3,
    };

    @SuppressWarnings("unused")
    private final String TAB_CONTACTS = "jp.kiroru_inc.samples.oredialer.ContactsAlias";
    private final String TAB_CALLLOG  = "jp.kiroru_inc.samples.oredialer.CallLogAlias";
    private final String TAB_DIALER   = "jp.kiroru_inc.samples.oredialer.DialerAlias";

    private ViewPager pager;
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        FragmentManager fm = getSupportFragmentManager();
        MyFragmentPagerAdapter adapter = new MyFragmentPagerAdapter(fm);
        
        pager = (ViewPager)findViewById(R.id.pager);
        pager.setOnPageChangeListener(new ViewPager.OnPageChangeListener() {
            public void onPageSelected(int position) {
                // ページが切り替わったらタブの表示を更新する
                updateTab(position);
            }
            public void onPageScrolled(int arg0, float arg1, int arg2) {}
            public void onPageScrollStateChanged(int arg0) {}
        });
        pager.setAdapter(adapter);

        // タブはラジオボタンで実装している
        RadioGroup rg = (RadioGroup)findViewById(R.id.rg);
        rg.setOnCheckedChangeListener(new OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(RadioGroup group, int checkedId) {
                RadioButton rb = (RadioButton)findViewById(checkedId);
                if (rb.isChecked()) {
                    // タブが押されたらページを切り替える
                    if (checkedId == R.id.rb_1) {
                        pager.setCurrentItem(0, true);
                    } else if (checkedId == R.id.rb_2) {
                        pager.setCurrentItem(1, true);
                    } else if (checkedId == R.id.rb_3) {
                        pager.setCurrentItem(2, true);
                    }
                }
            }
        });
        
        selectPage(getIntent());
    }
    
    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        
        selectPage(intent);
    }
    
    // 表示するページを指定する
    private void selectPage(Intent intent) {
        String componentName = intent.getComponent().getClassName();
        if (TAB_DIALER.equals(componentName)) {
            pager.setCurrentItem(0, true);
        } else if (TAB_CALLLOG.equals(componentName)) {
            pager.setCurrentItem(1, true);
        } else {
            pager.setCurrentItem(2, true);
        }
        if (TAB_DIALER.equals(componentName)) {
            DialPadFragment df = DialPadFragment.create();
            df.updateNumber(intent);
        }
    }

    // タブの表示を更新する
    private void updateTab(int page) {
        RadioGroup rg = (RadioGroup)findViewById(R.id.rg);
        rg.check(ids[page]);
    }
    
    private class MyFragmentPagerAdapter extends FragmentPagerAdapter {
        public MyFragmentPagerAdapter(FragmentManager fm) {
            super(fm);
        }
        
        @Override
        public Fragment getItem(int position) {
            Fragment f = null;
            if (position == 0) {
                f = DialPadFragment.create();
            } else if (position == 1) {
                f = CallLogFragment.create();
            } else if (position == 2) {
                f = ContactListFragment.create();
            }
            return f;
        }

        @Override
        public int getCount() {
            return ids.length;
        }
    }
}