package jp.kiroru_inc.samples.sample3;

import android.app.ListActivity;
import android.content.ContentUris;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Bundle;
import android.provider.ContactsContract.CommonDataKinds.Phone;
import android.provider.ContactsContract.CommonDataKinds.Photo;
import android.provider.ContactsContract.Data;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.CursorAdapter;
import android.widget.ImageView;
import android.widget.TextView;

/*
 * ダイアルパッドの呼出し
 */

public class MainActivity extends ListActivity {

    private String[] projection = {
            Phone._ID,           // 0
            Phone.PHOTO_ID,      // 1
            Phone.DISPLAY_NAME,  // 2
            Phone.NUMBER,        // 3
    };
    
    private static int INDEX_PHOTO_ID     = 1;
    private static int INDEX_DISPLAY_NAME = 2;
    private static int INDEX_NUMBER       = 3;
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        Cursor c = managedQuery(Phone.CONTENT_URI, projection, null, null, null);
        MyAdapter adapter = new MyAdapter(this, c);
        setListAdapter(adapter);
    
        getListView().setOnItemClickListener(new OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View v, int position, long id) {
                Uri uri = ContentUris.withAppendedId(Phone.CONTENT_URI, id);
                String[] projection = { Phone.NUMBER };
                Cursor c = getContentResolver().query(uri, projection, null, null, null);
                if (c.moveToFirst()) {
                    String number = (String)c.getString(0);
                    c.close();
                    dial(number);
                }                
            }
        });
    }
    
    private void dial(String number) {
        Uri telUri = Uri.parse("tel:" + number);
        Intent i = new Intent(Intent.ACTION_DIAL);
        i.setData(telUri);
        startActivity(i);
    }
    
    private class MyAdapter extends CursorAdapter {

        public MyAdapter(Context context, Cursor c) {
            super(context, c);
        }

        @Override
        public View newView(Context context, Cursor cursor, ViewGroup parent) {
            return LayoutInflater.from(context).inflate(R.layout.row, parent, false);
        }
        
        @Override
        public void bindView(View view, Context context, Cursor cursor) {
            // 写真
            ImageView iv = (ImageView) view.findViewById(R.id.image);
            byte[] photoData = null;
            String photoId = cursor.getString(INDEX_PHOTO_ID);
            if (photoId != null) {
                Uri uri = Uri.withAppendedPath(Data.CONTENT_URI, photoId);
                String[] projection = { Photo.PHOTO };
                Cursor c = context.getContentResolver().query(uri, projection, null, null, null);
                if (c.moveToFirst()) {
                    photoData = c.getBlob(0);
                }
                c.close();
            }
            if (photoData != null) {
                iv.setImageBitmap(BitmapFactory.decodeByteArray(photoData, 0, photoData.length));
            } else {
                iv.setImageResource(R.drawable.ic_contact_picture);
            }
            
            // 名前
            String text1 = cursor.getString(INDEX_DISPLAY_NAME);
            TextView tv1 = (TextView) view.findViewById(R.id.text1);
            tv1.setText(text1);

            // 電話番号
            String text2 = cursor.getString(INDEX_NUMBER);
            TextView tv2 = (TextView) view.findViewById(R.id.text2);
            tv2.setText(text2);
        }
    }
}