package jp.kiroru_inc.samples.sample6;

import java.text.SimpleDateFormat;

import android.app.ListActivity;
import android.content.ContentUris;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.provider.CallLog.Calls;
import android.provider.ContactsContract.Contacts;
import android.provider.ContactsContract.PhoneLookup;
import android.view.ContextMenu;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.ContextMenu.ContextMenuInfo;
import android.widget.AdapterView;
import android.widget.CursorAdapter;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.AdapterView.OnItemClickListener;

/*
 * 発着信履歴からのコンタクト詳細、ダイアルパッドの呼出し
 */

public class MainActivity extends ListActivity {

    private String[] projection = {
            Calls._ID,          // 0
            Calls.TYPE,         // 1
            Calls.DATE,         // 2
            Calls.CACHED_NAME,  // 3
            Calls.NUMBER,       // 4
    };
    
    private static int INDEX_TYPE         = 1;
    private static int INDEX_DATE         = 2;
    private static int INDEX_CACHED_NAME  = 3;
    private static int INDEX_NUMBER       = 4;
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        Cursor c = managedQuery(
                Calls.CONTENT_URI, projection, null, null, Calls.DEFAULT_SORT_ORDER);
        MyAdapter adapter = new MyAdapter(this, c);
        setListAdapter(adapter);
        registerForContextMenu(getListView());
        
        getListView().setOnItemClickListener(new OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View v, int position, long id) {
                Uri uri = ContentUris.withAppendedId(Calls.CONTENT_URI, id);
                String[] projection = { Calls.NUMBER };
                Cursor c = getContentResolver().query(uri, projection, null, null, null);
                if (c.moveToFirst()) {
                    String number = (String)c.getString(0);
                    c.close();
                    dial(number);
                }                
            }
        });
     }
    
    @Override
    public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
        menu.add(0, 0, 0, "ダイアルパッド");
        menu.add(0, 1, 1, "詳細");
        menu.add(0, 2, 2, "コンタクトに登録又は追加");
    };

    @Override
    public boolean onContextItemSelected(MenuItem item) {
        long id = ((AdapterContextMenuInfo)item.getMenuInfo()).id;
        int menuId = item.getItemId();
        if (0 <= menuId && menuId <= 2) {
            Uri uri = ContentUris.withAppendedId(Calls.CONTENT_URI, id);
            String[] projection = { Calls.NUMBER };
            Cursor c = getContentResolver().query(uri, projection, null, null, null);
            if (c.moveToFirst()) {
                String number = (String)c.getString(0);
                c.close();
                if (number != null && !number.equals("") && !number.startsWith("-")) {
                    if (menuId == 0) {
                        dial(number);
                    } else if (menuId == 1) {
                        view(number);
                    } else {
                        insert_or_edit(number);
                    }
                }
            }                
            return true;
        }
        return false;
    }
    
    private void dial(String number) {
        Uri telUri = Uri.parse("tel:" + number);
        Intent i = new Intent(Intent.ACTION_DIAL);
        i.setData(telUri);
        i.putExtra("number", number);
        startActivity(i);
    }
    
    private void view(String number) {
        Uri phoneLookupUri = Uri.withAppendedPath(PhoneLookup.CONTENT_FILTER_URI, Uri.encode(number));
        String[] projection = { PhoneLookup.LOOKUP_KEY };
        Cursor c = getContentResolver().query(phoneLookupUri, projection, null, null, null);
        if (c.moveToFirst()) {
            String lookupKey = (String)c.getString(0);
            c.close();
            Uri lookupUri = Uri.withAppendedPath(Contacts.CONTENT_LOOKUP_URI, lookupKey);
            Intent i = new Intent(Intent.ACTION_VIEW);
            i.setData(lookupUri);
            startActivity(i);
        }
    }
    
    private void insert_or_edit(String number) {
        Intent i = new Intent(Intent.ACTION_INSERT_OR_EDIT);
        i.setType(Contacts.CONTENT_ITEM_TYPE);
        i.putExtra("phone", number);
        startActivity(i);
    }
    
    private class MyAdapter extends CursorAdapter {

        public MyAdapter(Context context, Cursor c) {
            super(context, c);
        }

        @Override
        public View newView(Context context, Cursor cursor, ViewGroup parent) {
            return LayoutInflater.from(context).inflate(R.layout.row, parent, false);
        }
        
        @Override
        public void bindView(View view, Context context, Cursor cursor) {
            // タイプ
            int resId;
            int type = cursor.getInt(INDEX_TYPE);
            if (type == Calls.INCOMING_TYPE) {
                resId = android.R.drawable.sym_call_incoming;
            } else if (type == Calls.OUTGOING_TYPE) {
                resId = android.R.drawable.sym_call_outgoing;
            } else { // (type == Calls.MISSED_TYPE) 
                resId = android.R.drawable.sym_call_missed;
            }
            ImageView iv = (ImageView)view.findViewById(R.id.image);
            iv.setImageResource(resId);
            
            // 時刻
            long date = cursor.getLong(INDEX_DATE);
            SimpleDateFormat sdf1 = new SimpleDateFormat();
            sdf1.applyPattern("yyyy/MM/dd hh:mm");
            String text1 = sdf1.format(date);
            TextView tv1 = (TextView) view.findViewById(R.id.text1);
            tv1.setText(text1);
            
            // 名前
            String text2 = cursor.getString(INDEX_CACHED_NAME);
            TextView tv2 = (TextView) view.findViewById(R.id.text2);
            tv2.setText(text2);

            // 電話番号
            String text3 = cursor.getString(INDEX_NUMBER);
            TextView tv3 = (TextView) view.findViewById(R.id.text3);
            tv3.setText(text3);
        }
    }
}